@@ -398,9 +398,12 @@ struct context_2
   in_addr_t push_ifconfig_local;
   in_addr_t push_ifconfig_remote_netmask;
 
+  /* client authentication state */
 # define CAS_SUCCEEDED 0
 # define CAS_PENDING   1
 # define CAS_FAILED    2
+# define CAS_PARTIAL   3 /* at least one client-connect script/plugin
+			    succeeded while a later one in the chain failed */
   int context_auth;
 #endif
 
@@ -424,7 +427,7 @@ struct context
   bool first_time;
 
   /* used by multi-client code to lock the context */
-  //MUTEX_DEFINE (mutex);
+  /*MUTEX_DEFINE (mutex);*/
 
   /* context modes */
 # define CM_P2P            0 /* standalone point-to-point session or client */
